unit filterDynamic;
(* ***** BEGIN LICENSE BLOCK *****
  * Copyright (C) 2004 Durand Emmanuel
  * Copyright (C) 2004 Burgel Eric
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Contact :
  *   filters@edurand.com
  *   filters@burgel.com
  *
  * ***** END LICENSE BLOCK ***** *)

{
  edurand (filters@edurand.com)
}

interface

uses
  filter, fparameters, image, SysUtils, Graphics;

type
  TFilterDynamic = class(TFilter)
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure Run(); override;
  private
    parameterImageIn: TParameterImage;
    ParameterImageOut: TParameterImage;
    ParameterImageAverage : TParameterInteger;
    ParameterImageRMS : TParameterInteger;
    imgIn, imageOut: PBitmap32;
    procedure _run();
  end;

implementation

uses
  imageIO, Math, divers;

constructor TFilterDynamic.Create;
begin
  inherited;
  parameterImageIn := addParameterImage('INIMAGE', 'explication...');
  ParameterImageOut := addParameterImage('OUTIMAGE', 'outImage');
  parameterImageAverage := addOutputParameterInteger('AVERAGE', 'average of image');
  parameterImageRMS := addOutputParameterInteger('RMS', 'RMS of image');
  setParameterImage('OUTIMAGE', imageOut);
end;

destructor TFilterDynamic.Destroy;
begin
  inherited;
  freeImage(imageOut);
end;

procedure TFilterDynamic.Run();
begin
  imgIn := parameterImageIn.image;
  if (imgIn <> nil) then
  begin
    _run();
  end;
end;

procedure TFilterDynamic._run();
var
  PSrc, PDest: PColor32Array;
  maxX, maxY : Integer;
  x, y, N, MinG, MaxG, tLevel, Average: Integer;
  imgOut: PBitmap32;
  color: TColor32;
  Red, Green, Blue: byte;
  RMS : cardinal;
begin
  imgOut := ParameterImageOut.image;

  maxX := scanWidth(imgIn) - 1;
  maxY := scanHeight(imgIn) - 1;

  MaxG := 0;
  MinG := 255 * 3;

  { ****************************************************************************** }

  if (maxX > 0) and (maxY > 0) then // determine min max
  begin
    for y := 0 to maxY do
    Begin
      PSrc := scanLine(imgIn, y);
      for x := 0 to maxX do
      Begin
        Red := (PSrc^[0] shr 16) and $FF;
        Green := (PSrc^[0] shr 8) and $FF;
        Blue := PSrc^[0] and $FF;
        tLevel := Red + Blue + Green; // determine grey shade
        if tLevel < MinG then MinG := tLevel;
        if tLevel > MaxG then MaxG := tLevel;
        Inc(PSrc);
      end;
    end;
  end;

  { ****************************************************************************** }

  N := 0;
  RMS := 0;
  Average := 0;

  if (maxX > 0) and (maxY > 0) then
  begin
    for y := 0 to maxY do
    Begin
      PSrc := scanLine(imgIn, y);
      PDest := scanLine(imgOut, y);
      for x := 0 to maxX do
      begin
        Red := (PSrc^[0] shr 16) and $FF;
        Green := (PSrc^[0] shr 8) and $FF;
        Blue := PSrc^[0] and $FF;

        tLevel := Red + Blue + Green; // determine grey shade

        if (tLevel - MinG) > 0 then
          tLevel := trunc(((tLevel - MinG) * ((255 * 3) / (MaxG - MinG))) / 3)
        else
          tLevel := 0;

        if tLevel > 255 then tlevel := 255;
        if tLevel < 0 then tlevel := 0;

        Inc(N);

        Average := Average + tLevel;
        RMS := RMS + Cardinal((tLevel * tLevel));  // calc squares
        Color := ((tLevel) shl 16) or (tLevel shl 8) or (tLevel);
        PDest^[0] := color;
        Inc(PDest);
        Inc(PSrc);
      end;
    end;
  end;

  { ****************************************************************************** }

  ParameterImageRMS.Value := Round(Sqrt(RMS / N));  // calc RMS
  ParameterImageAverage.Value := Average div N;
end;

end.
