unit FilterWindow;

interface

uses
  filter, fparameters, image;

type
  TFilterWindow = class(TFilter)
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure Run(); override;
  private
    parameterImageIn, parameterImageOut: TParameterImage;
    parameterImageTop : TParameterInteger;
    parameterImageBottom : TParameterInteger;
    parameterImageLeft : TParameterInteger;
    parameterImageRight : TParameterInteger;
    ImageIn, ImageOut : PBitmap32;
    Left, Right, Top, Bottom : cardinal;
    procedure _run();
  end;

implementation

uses
  imageIO;

constructor TFilterWindow.Create;
begin
  inherited;
  parameterImageIn := addParameterImage('INIMAGE', 'input image');
  parameterImageOut := addParameterImage('OUTIMAGE', 'result image');
  parameterImageTop := addParameterInteger('TOP', 'top greater than bottom', 0, 10000, 0);
  parameterImageBottom := addParameterInteger('BOTTOM', 'bottom less than top', 0, 10000, 0);
  parameterImageLeft := addParameterInteger('LEFT', 'left less than right', 0, 10000, 0);
  parameterImageRight := addParameterInteger('RIGHT', 'right greater than left', 0, 10000, 0);
end;

destructor TFilterWindow.Destroy;
begin
  inherited;
End;

procedure TFilterWindow.Run();
begin
  ImageIn := parameterImageIn.image;
  ImageOut := parameterImageOut.image;
  Left := ParameterImageLeft.Value;
  Right := ParameterImageRight.Value;
  Top := ParameterImageTop.Value;
  Bottom := ParameterImageBottom.Value;
  if (ImageIn <> nil) and (ImageOut <> nil) then
  begin
    _run();
  end;
end;

procedure TFilterWindow._run();
var
  pSrc, PDest: PColor32Array;
  X, Y, OutWidth, OutHeight: cardinal;
begin
  if Left > Right then  // keep correct order
  begin
    X := Right;
    Right := Left;
    Left := X;
  end;

  if Top < Bottom then  // keep correct order
  begin
    X := Bottom;
    Bottom := Top;
    Top := X;
  end;

  if Left > scanWidth(ImageIn) then Left := scanWidth(ImageIn);
  if Right > scanWidth(ImageIn) then Right := scanWidth(ImageIn);
  if Top > scanHeight(ImageIn) then Top := scanHeight(ImageIn);
  if Bottom > scanHeight(ImageIn) then Bottom := scanHeight(ImageIn);

  OutWidth := Right - Left;
  OutHeight := Top - Bottom;
  ImageOut.Width := Succ(OutWidth);
  ImageOut.Height := Succ(OutHeight);

  for Y := 0 to OutHeight do
  begin
    pSrc := scanLine(ImageIn, Y + Bottom);
    PDest := scanLine(ImageOut, Y);
    for X := 0 to OutWidth do
    begin
      PDest^[X] := pSrc^[X + Left];
    end;
  end;
end;

end.
