unit ImportPlaceFile;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, Grids, UiTypes;

type
  TFormImportPlaceFile = class(TForm)
    StringGrid1: TStringGrid;
    ButtonOk: TBitBtn;
    ButtonClose: TBitBtn;
    FileOpenDialogPlace: TOpenDialog;
    ListBoxInput: TListBox;
    BurronOpenFile: TBitBtn;
    procedure BurronOpenFileClick(Sender: TObject);
    procedure ButtonOkClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    function OpenFile: boolean;
    procedure ImportFile;
    procedure UpdateStringList;
    Procedure ExtractCommands(T: string);
    Function ExtractValue(T: string): Double;
    procedure Sort_Parts_List; { sort the parts list by footprint & rotation }
  public
    { Public declarations }
  end;

var
  FormImportPlaceFile: TFormImportPlaceFile;

implementation

{$R *.dfm}

uses
  StrUtils, Main, Settings;

const
  Max_Parts = 10000;
  CrLf: string = #$0d + #$0a;
  Tab: Char = #9;

type
  PartRec = record
    Designator: String;
    Pattern: String;
    X: Double;
    Y: Double;
    Layer: String;
    Rotation: Double;
    Comment: String;
  end;

var
  XPos, YPos: Double;
  ItemString: string;
  Command: array [1 .. 99] of string;
  Commands: integer;
  Parts: array of PartRec;
  Number_Of_Parts: integer;
  Metric: boolean;

procedure TFormImportPlaceFile.BurronOpenFileClick(Sender: TObject);
begin
  StringGrid1.RowCount := 0;
  StringGrid1.ColCount := 0;
  if OpenFile then
  begin
    ImportFile;
    UpdateStringList;
    ButtonOk.Enabled := true;
  end
  else
    ButtonOk.Enabled := false;
end;

function TFormImportPlaceFile.OpenFile: boolean;
Var
  S, FName: string;
  F: TextFile;

begin
  { validate file path }

  FileOpenDialogPlace.InitialDir := Data_Directory;
  FileOpenDialogPlace.FileName := '';
  if FileOpenDialogPlace.Execute then
  begin
    FName := FileOpenDialogPlace.FileName;
    AssignFile(F, FName);
    ListBoxInput.Clear;
    try
      Reset(F);
      if not Eof(F) then
        repeat
          ReadLn(F, S); { read data }
          ListBoxInput.AddItem(S, @Self);
        until Eof(F);
      CloseFile(F);
      Result := true;
    Except
      MessageDlg('File Error.', mtInformation, [mbOK], 0);
      CloseFile(F);
      Result := false;
    end;
  end
  else
    Result := false;
end;

Function TFormImportPlaceFile.ExtractValue(T: string): Double;
var
  X: integer;
  S: string;
  C: Char;
  V: Double;
begin
  if Length(T) > 0 then
  begin
    S := ''; { clear workspace }
    for X := 1 to Length(T) do { strip all non numeric characters }
    begin
      C := T[X];
      case C of
        '0' .. '9', '-', '.':
          S := S + C;
      end;
    end;

    if Length(S) > 0 then { if string is not zero length }
    begin
      V := StrToFloat(S);
      Result := V;
    end
    else
      Result := 0; { zero length string }
  end
  else
    Result := 0; { zero length string }
end;

procedure TFormImportPlaceFile.FormShow(Sender: TObject);
begin
  StringGrid1.RowCount := 0;
  StringGrid1.ColCount := 0;
end;

procedure TFormImportPlaceFile.ButtonOkClick(Sender: TObject);
var
  X: integer;
begin
  // file has now been imported and is ready for transfer
  if Number_Of_Parts > 0 then
  begin
    PartCount := Number_Of_Parts;
    SetLength(PartList, PartCount);

    for X := 0 to PartCount - 1 do
    begin
      PartList[X].Designator := Parts[X].Designator;
      PartList[X].Comment := Parts[X].Comment + ' ' + Parts[X].Pattern;
      PartList[X].X := Parts[X].X;
      PartList[X].Y := Parts[X].Y;
      PartList[X].Angle := Parts[X].Rotation;
      PartList[X].Feeder := 0;
      PartList[X].Enabled := true;
    end;
    BoardMetric := Metric; // update units format
    ProjectEdited := true;

    FormMain.UpdatePartList;
    FormMain.UpdateStatus;
    FormMain.UpdateButtons;
  end;
  ModalResult := mrOk;
end;

Procedure TFormImportPlaceFile.ExtractCommands(T: string);
var
  X: integer;
  S: string;
  C: Char;
  Done: boolean;

begin
  for X := 1 to 99 do { erase workspace }
    Command[X] := '';

  X := 0;
  Commands := 0;
  S := '';
  Done := false;

  if T <> '' then { if data to process }
  begin
    Repeat
      Application.ProcessMessages;
      Inc(X);
      C := T[X]; { get character }

      case C of
        ',', ' ', #9: { , tab or space command separator }
          begin
            if S <> '' then
            begin
              Inc(Commands);
              Command[Commands] := S; { save current command }
              S := '';
            end;
          end;
        '"':
          begin { ignore quote marks }

          end;

      else
        S := S + C; { else add to string }
      end;

      if X >= Length(T) then
      begin
        Inc(Commands);
        Command[Commands] := S; { save current command }
        Done := true;
      end;

    Until Done; { repeat until finished }
  end;
end;

procedure TFormImportPlaceFile.ImportFile;
var
  X, Y, Z: integer;
begin
  Number_Of_Parts := 0;
  SetLength(Parts, Number_Of_Parts); // reset array

  if (ListBoxInput.Items.Count > 0) then { if items exist the convert }
  begin
    XPos := 0;
    YPos := 0;

    { *************** process each line assuming one command per line ************** }

    for X := 1 to ListBoxInput.Items.Count do
    begin
      Application.ProcessMessages;
      ItemString := ListBoxInput.Items.Strings[Pred(X)]; { get line to process }
      begin
        ExtractCommands(ItemString);

        if Commands >= 10 then { process parameter from 2 onward until mil or mm field }
        begin
          for Y := 1 to 10 do
          begin
            if (RightStr(Command[3], 3) <> 'mil') and (RightStr(Command[3], 2) <> 'mm') then
            begin
              Command[2] := Command[2] + ' ' + Command[3];
              for Z := 3 to 98 do
                Command[Z] := Command[Succ(Z)]; { shift commands down by one }
            end;
          end;
        end;

        { *********** test for valid line *********** }

        if (RightStr(Command[3], 3) = 'mil') and (RightStr(Command[4], 3) = 'mil') and (RightStr(Command[5], 3) = 'mil') and (RightStr(Command[6], 3) = 'mil')
          and (RightStr(Command[7], 3) = 'mil') and (RightStr(Command[8], 3) = 'mil') then { test for valid imperial line }
        begin
          Metric := false;

          Inc(Number_Of_Parts);
          SetLength(Parts, Number_Of_Parts);

          Parts[Number_Of_Parts - 1].Designator := Command[1];
          Parts[Number_Of_Parts - 1].Pattern := Command[2];
          Parts[Number_Of_Parts - 1].X := ExtractValue(Command[3]) / 1000 * 25.4; // convert to metric
          Parts[Number_Of_Parts - 1].Y := ExtractValue(Command[4]) / 1000 * 25.4; // convert to metric
          Parts[Number_Of_Parts - 1].Layer := Command[9];
          Parts[Number_Of_Parts - 1].Rotation := ExtractValue(Command[10]);
          Parts[Number_Of_Parts - 1].Comment := Command[11];

          if Parts[Number_Of_Parts - 1].Rotation >= 360 then
            Parts[Number_Of_Parts - 1].Rotation := Parts[Number_Of_Parts - 1].Rotation - 360
          else if Parts[Number_Of_Parts - 1].Rotation <= -360 then
            Parts[Number_Of_Parts - 1].Rotation := Parts[Number_Of_Parts - 1].Rotation + 360;

        end
        else if (RightStr(Command[3], 2) = 'mm') and (RightStr(Command[4], 2) = 'mm') and (RightStr(Command[5], 2) = 'mm') and (RightStr(Command[6], 2) = 'mm')
          and (RightStr(Command[7], 2) = 'mm') and (RightStr(Command[8], 2) = 'mm') then { test for valid metric line }
        begin
          Metric := true;

          Inc(Number_Of_Parts);
          SetLength(Parts, Number_Of_Parts);

          Parts[Number_Of_Parts - 1].Designator := Command[1];
          Parts[Number_Of_Parts - 1].Pattern := Command[2];
          Parts[Number_Of_Parts - 1].X := ExtractValue(Command[3]);
          Parts[Number_Of_Parts - 1].Y := ExtractValue(Command[4]);
          Parts[Number_Of_Parts - 1].Layer := Command[9];
          Parts[Number_Of_Parts - 1].Rotation := ExtractValue(Command[10]);
          Parts[Number_Of_Parts - 1].Comment := Command[11];

          if Parts[Number_Of_Parts - 1].Rotation >= 360 then
            Parts[Number_Of_Parts - 1].Rotation := Parts[Number_Of_Parts - 1].Rotation - 360
          else if Parts[Number_Of_Parts - 1].Rotation <= -360 then
            Parts[Number_Of_Parts - 1].Rotation := Parts[Number_Of_Parts - 1].Rotation + 360;

        end;
      end
    end;

    // ************* sort the parts list *************

    Sort_Parts_List; // Sort the resulting list
  end;
end;

procedure TFormImportPlaceFile.UpdateStringList;
var
  X: integer;
begin // file has now been imported and is ready for transfer
  StringGrid1.RowCount := 1;
  StringGrid1.ColCount := 6;
  StringGrid1.Cells[0, 0] := 'Designator';
  StringGrid1.Cells[1, 0] := 'Comment';
  StringGrid1.Cells[2, 0] := 'Pattern';
  if Metric then
  begin
    StringGrid1.Cells[3, 0] := 'X mm';
    StringGrid1.Cells[4, 0] := 'Y mm';
  end
  else
  begin
    StringGrid1.Cells[3, 0] := 'X in';
    StringGrid1.Cells[4, 0] := 'Y in';
  end;

  StringGrid1.Cells[5, 0] := 'Angle';

  if Number_Of_Parts > 0 then
  begin

    StringGrid1.RowCount := Succ(Number_Of_Parts);

    for X := 0 to Number_Of_Parts - 1 do
    begin
      StringGrid1.Cells[0, Succ(X)] := Parts[X].Designator;
      StringGrid1.Cells[1, Succ(X)] := Parts[X].Comment;
      StringGrid1.Cells[2, Succ(X)] := Parts[X].Pattern;
      if Metric then
      begin
        StringGrid1.Cells[3, Succ(X)] := FloatToStrF(Parts[X].X, ffFixed, 10, 4);
        StringGrid1.Cells[4, Succ(X)] := FloatToStrF(Parts[X].Y, ffFixed, 10, 4);
      end
      else
      begin
        StringGrid1.Cells[3, Succ(X)] := FloatToStrF(Parts[X].X / 25.4, ffFixed, 10, 4);
        StringGrid1.Cells[4, Succ(X)] := FloatToStrF(Parts[X].Y / 25.4, ffFixed, 10, 4);
      end;
      StringGrid1.Cells[5, Succ(X)] := FloatToStrF(Parts[X].Rotation, ffFixed, 10, 3);
    end;
  end;
end;

procedure TFormImportPlaceFile.Sort_Parts_List; // sort the parts list by footprint & rotation

  Function PartRecCompare(A, B: PartRec): boolean;

    function PadString(S: string): String;
    begin
      Result := LeftStr(S + '                                                  ', 50);
    end;

  Var
    Pa, Pb: String;
  begin
    Pa := PadString(A.Pattern) + PadString(A.Comment) + PadString(FloatToStrF(A.Rotation, ffFixed, 12, 3));
    Pb := PadString(B.Pattern) + PadString(B.Comment) + PadString(FloatToStrF(B.Rotation, ffFixed, 12, 3));
    Result := (Pa > Pb);
  end;

  Procedure SwapRec(Var A, B: PartRec);
  Var
    C: PartRec;
  Begin
    C := A;
    A := B;
    B := C;
  end;

  procedure QuickSortNames(var Vector: array of PartRec);
  { QuickSort for array }
  var
    First, Last: Longint;

    procedure Partition(var Vector: array of PartRec; First, Last: Longint);
    var
      Right, Left: Longint;
      V: PartRec;
    begin
      V := Vector[(First + Last) shr 1];
      Right := First;
      Left := Last;
      repeat
        while PartRecCompare(V, Vector[Right]) do
          Inc(Right);
        while PartRecCompare(Vector[Left], V) do
          Dec(Left);
        if (Right <= Left) then
        begin
          SwapRec(Vector[Right], Vector[Left]);
          Inc(Right);
          Dec(Left);
        end;
      until Right > Left;
      if (First < Left) then
        Partition(Vector, First, Left);
      if (Right < Last) then
        Partition(Vector, Right, Last);
    end;

  begin
    First := 0; { must use values - 1 }
    Last := Pred(Number_Of_Parts);
    if First < Last then
      Partition(Vector, First, Last)
  end;

begin
  if Number_Of_Parts > 1 then
    QuickSortNames(Parts); { QuickSort parts }
end;

end.
