unit filterCanny;
(* ***** BEGIN LICENSE BLOCK *****
  * Copyright (C) 2004 Durand Emmanuel
  * Copyright (C) 2004 Burgel Eric
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Contact :
  *   filters@edurand.com
  *   filters@burgel.com
  *
  * ***** END LICENSE BLOCK ***** *)

{
  edurand (filters@edurand.com)
}

{
  use 'OpenCv' implementation
}

{
  doc : http://www.pages.drexel.edu/~weg22/can_tut.html
}

interface

uses
  filter, fparameters, image;

type
  TFilterCanny = class(TFilter)
  public
    constructor Create; override;
    procedure Run(); override;
  private
    parameterImageIn, parameterImageOut: TParameterImage;
    parameterThreshold1: TParameterInteger;
    parameterThreshold2: TParameterInteger;
  end;

implementation

uses
  imageIO, wrapper_opencv;

constructor TFilterCanny.Create;
begin
  inherited;
  parameterImageIn := addParameterImage('INIMAGE', 'input image');
  parameterImageOut := addParameterImage('OUTIMAGE', 'result image');
  parameterThreshold1 := addParameterInteger('THRESHOLD1', 'threshold1', 0, 255, 100);
  parameterThreshold2 := addParameterInteger('THRESHOLD2', 'threshold1', 0, 255, 100);
end;

procedure TFilterCanny.Run();
begin
  if (parameterImageIn.image <> nil) and (parameterImageOut.image <> nil) then
  begin
    wrapper_opencv.filterCvCanny(parameterImageIn.image, parameterImageOut.image, parameterThreshold1.Value,
      parameterThreshold2.Value);
  end;
end;

end.
